from __future__ import print_function

from collections import defaultdict

from moneywagon import ExchangeUniverse
from moneywagon.services import Service

def all_balances(currency, services=None, verbose=False, timeout=None):
    """
    Get balances for passed in currency for all exchanges.
    """
    balances = {}
    if not services:
        services = [
            x(verbose=verbose, timeout=timeout)
            for x in ExchangeUniverse.get_authenticated_services()
        ]

    for e in services:
        try:
            balances[e] = e.get_exchange_balance(currency)
        except NotImplementedError:
            if verbose:
                print(e.name, "balance not implemented")
        except Exception as exc:
            if verbose:
                print(e.name, "failed:", exc.__class__.__name__, str(exc))

    return balances

def total_exchange_balances(services=None, verbose=None, timeout=None):
    """
    Returns all balances for all currencies for all exchanges
    """
    balances = defaultdict(lambda: 0)
    if not services:
        services = [
            x(verbose=verbose, timeout=timeout)
            for x in ExchangeUniverse.get_authenticated_services()
        ]

    for e in services:
        try:
            more_balances = e.get_total_exchange_balances()
            for code, bal in more_balances.items():
                balances[code] += bal
        except NotImplementedError:
            if verbose:
                print(e.name, "total balance not implemented")
        except Exception as exc:
            if verbose:
                print(e.name, "failed:", exc.__class__.__name__, str(exc))

    return balances

def transfer_balance_on_exchange(currency, from_ex, to_ex, percent=None, amount=None, verbose=False):
    if not isinstance(from_ex, Service): # if class passed in, instantiate
        from_ex = from_ex(verbose=verbose)
    if not isinstance(to_ex, Service):
        to_ex = to_ex(verbose=verbose)

    if not amount and not percent:
        raise Exception("One of `amount` or `percent` required.")
    if percent and amount:
        raise Exception("Either `amount` or `percent`, not both")

    if percent:
        balance = from_ex.get_exchange_balance(currency)
        amount = balance * (percent / 100)
        if verbose:
            print("Sending %.2f%% of %.8f which is %.8f" % (percent, balance, amount))
    to_address = to_ex.get_deposit_address(currency)
    if verbose:
        print("to address: %s" % to_address)
    return from_ex.initiate_withdraw(currency, amount, to_address)


class MultiOrderBook(object):

    def __init__(self, services=None, verbose=False):
        from moneywagon import EXCHANGE_SERVICES
        self.services = services or [x(verbose=verbose) for x in EXCHANGE_SERVICES]
        self.got_orderbook_services = []
        self.verbose = verbose

    def get(self, crypto, fiat, trim=False, trim_crypto=None, trim_fiat=None):
        services = None
        if trim:
            trim_crypto, trim_fiat = True, True
        if trim_fiat is True:
            trim_fiat = all_balances(fiat, services=self.services)
        if trim_crypto is True:
            if trim_fiat:
                services = trim_fiat.keys()
            else:
                services = self.services
            trim_crypto = all_balances(crypto, services=services)

        if trim_crypto or trim_fiat:
            services = self._services_from_balances(trim_crypto, trim_fiat)
        elif not services:
            services = self.services

        combined = {'bids': [], 'asks': []}
        for service in services:
            try:
                book = service.get_orderbook(crypto, fiat)
                combined = self._combine_orderbook(combined, book, service)
                self.got_orderbook_services.append(service)
            except NotImplementedError:
                pass
            except Exception as exc:
                print("%s orderbook failed: %s: %s" % (service.name, exc.__class__, str(exc)))

        if trim_fiat:
            combined['asks'] = self._trim(combined['asks'], trim_fiat, side='fiat')

        if trim_crypto:
            combined['bids'] = self._trim(combined['bids'], trim_crypto, side='crypto')

        return combined

    def _combine_orderbook(self, combined_book, new_book, new_book_service):
        for side in ['bids', 'asks']:
            for order in new_book[side]:
                with_name = (order[0], order[1], new_book_service)
                combined_book[side].append(with_name)

        combined_book['bids'] = sorted(combined_book['bids'], key=lambda x: x[0], reverse=True)
        combined_book['asks'] = sorted(combined_book['asks'], key=lambda x: x[0])

        return combined_book

    def _trim(self, book, balances, side):
        """
        >>> m = MultiOrderbook()
        >>> book = [
            [7800, 1.1, GDAX()],
            [7805, 3.2, Poloniex()],
            [7810, 0.3, GDAX()],
            [7900, 7.0, GDAX()]
        ]
        >>> m._trim(book, {GDAX(): 1.2, Poloniex(): 1.0}, 'crypto')
        [[7800, 1.1, <Service: GDAX (0 in cache)>],
         [7805, 1.0, <Service: Poloniex (0 in cache)>],
         [7810, 0.1, <Service: GDAX (0 in cache)>]]
        """
        new_book = []
        for service, balance in balances.items():
            accumulation = 0
            for order in book:
                if order[2].name == service.name:
                    if side == 'crypto':
                        size = order[1]
                    else:
                        size = order[1] * order[0] # convert to fiat

                    if size + accumulation <= balance:
                        new_book.append(order)
                        accumulation += size
                    else:
                        remaining = balance - accumulation
                        new_book.append([order[0], float("%.8f" % remaining), order[2]])
                        break

        return sorted(new_book, key=lambda x: x[0], reverse=side == 'fiat')

    def _services_from_balances(self, bal1, bal2):
        return (
            set(s for s,b in bal1.items() if b > 0) |
            set(s for s,b in bal2.items() if b > 0)
        )



from __future__ import print_function
import sys
from binascii import hexlify
from tabulate import tabulate
import hashlib

from base58 import b58decode_check, b58encode_check

from .core import (
    AutoFallbackFetcher, enforce_service_mode, get_optimal_services, get_magic_bytes,
    RevertToPrivateMode, CurrencyNotSupported, NoService, NoServicesDefined, Service
)
from .historical_price import Quandl
from .crypto_data import crypto_data
from bitcoin import sha256, pubtoaddr, privtopub, encode_privkey, encode_pubkey, privkey_to_address
from socketIO_client import SocketIO
from moneywagon.services import _get_all_services

ALL_SERVICES = _get_all_services()
EXCHANGE_SERVICES = _get_all_services(just_exchange=True)

is_py2 = False
if sys.version_info <= (3,0):
    is_py2 = True

class CompositeResponse(object):
    def __init__(self, service1, service2):
        self.service1 = service1
        self.service2 = service2

    def json(self):
        return {
            self.service1.name: self.service1.last_raw_response.json(),
            self.service2.name: self.service2.last_raw_response.json(),
        }

class CompositeService(object):
    """
    This object mimicks the Service class and is used when the price fetcher has
    to fetch two different price sources. This object is only used when invoking
    `report_services`.
    """
    def __init__(self, services1, services2, via):
        service1 = services1[0]
        service2 = services2[0]

        self.name = "%s -> %s (via %s)" % (
            service1.name, getattr(service2, 'name', 'None'), via.upper()
        )
        self.last_url = "%s, %s" % (service1.last_url, getattr(service2, 'last_url', None))
        self.last_raw_response = CompositeResponse(service1, service2)
        self.service_id = "%d+%d" % (service1.service_id, service2.service_id)

    def __repr__(self):
        return "<Composite Service: %s>" % self.name


def _try_price_fetch(services, args, modes):
    try:
        return enforce_service_mode(
            services, CurrentPrice, args, modes=modes
        )
    except NoService as exc:
        return exc

def get_current_price(crypto, fiat, services=None, convert_to=None, helper_prices=None, **modes):
    """
    High level function for getting current exchange rate for a cryptocurrency.
    If the fiat value is not explicitly defined, it will try the wildcard service.
    if that does not work, it tries converting to an intermediate cryptocurrency
    if available.
    """
    fiat = fiat.lower()
    args = {'crypto': crypto, 'fiat': fiat, 'convert_to': convert_to}

    if not services:
        services = get_optimal_services(crypto, 'current_price')

    if fiat in services:
        # first, try service with explicit fiat support
        try_services = services[fiat]
        result = _try_price_fetch(try_services, args, modes)
        if not isinstance(result, Exception):
            return result

    if '*' in services:
        # then try wildcard service
        try_services = services['*']
        result = _try_price_fetch(try_services, args, modes)
        if not isinstance(result, Exception):
            return result

    def _do_composite_price_fetch(crypto, convert_crypto, fiat, helpers, modes):
        before = modes.get('report_services', False)
        modes['report_services'] = True
        services1, converted_price = get_current_price(crypto, convert_crypto, **modes)
        if not helpers or convert_crypto not in helpers[fiat]:
            services2, fiat_price = get_current_price(convert_crypto, fiat, **modes)
        else:
            services2, fiat_price = helpers[fiat][convert_crypto]

        modes['report_services'] = before

        if modes.get('report_services', False):
            #print("composit service:", crypto, fiat, services1, services2)
            serv = CompositeService(services1, services2, convert_crypto)
            return [serv], converted_price * fiat_price
        else:
            return converted_price * fiat_price

    all_composite_cryptos = ['btc', 'ltc', 'doge', 'uno']
    if crypto in all_composite_cryptos: all_composite_cryptos.remove(crypto)
    for composite_attempt in all_composite_cryptos:
        if composite_attempt in services and services[composite_attempt]:
            result = _do_composite_price_fetch(
                crypto, composite_attempt, fiat, helper_prices, modes
            )
            if not isinstance(result, Exception):
                return result

    raise result

def get_fiat_exchange_rate(from_fiat, to_fiat):
    from moneywagon.services import FreeCurrencyConverter
    c = FreeCurrencyConverter()
    return c.get_fiat_exchange_rate(from_fiat, to_fiat)

def get_address_balance(crypto, address=None, addresses=None, services=None, **modes):
    if not services:
        services = get_optimal_services(crypto, 'address_balance')

    args = {'crypto': crypto}

    if address:
        args['address'] = address
    elif addresses:
        args['addresses'] = addresses
    else:
        raise Exception("Either address or addresses but not both")

    results = enforce_service_mode(
        services, AddressBalance, args, modes=modes
    )

    if modes.get('private') and addresses:
        results['total_balance'] = sum(results.values())

    if modes.get('private') and modes.get('report_services', False):
        # private mode does not return services (its not practical),
        # an empty list is returned in its place to simplify the API.
        return [], results

    return results

def get_historical_transactions(crypto, address=None, addresses=None, services=None, **modes):
    if not services:
        services = get_optimal_services(crypto, 'historical_transactions')

    kwargs = {'crypto': crypto}
    if addresses:
        kwargs['addresses'] = addresses
    if address:
        kwargs['address'] = address

    try:
        txs = enforce_service_mode(
            services, HistoricalTransactions, kwargs, modes=modes
        )
    except RevertToPrivateMode:
        # no services implement get_historical_transactions_multi...
        modes['private'] = 1
        if modes.get('verbose'):
            print("Can't make with single API call. Retrying with private mode")
        txs = enforce_service_mode(
            services, HistoricalTransactions, kwargs, modes=modes
        )

    if modes.get('private'):
        # private mode returns items indexed by address, this only makes sense to do
        # for address balance, so remove it here
        just_txs = []
        [just_txs.extend(x) for x in txs.values()]
        txs = sorted(just_txs, key=lambda tx: tx['date'], reverse=True)

        no_duplicates = []
        all_txids = []
        for tx in txs:
            # private mode may return duplicate txs, remove them here.
            if tx['txid'] in all_txids:
                continue
            all_txids.append(tx['txid'])
            no_duplicates.append(tx)

        txs = no_duplicates

        if modes.get('report_services', False):
            # private mode does not return services (its not practical),
            # an empty list is returned in its place to simplify the API.
            return [], txs



    return txs

def get_single_transaction(crypto, txid, services=None, **modes):
    if not services:
        services = get_optimal_services(crypto, 'single_transaction')

    return enforce_service_mode(
        services, SingleTransaction, {'crypto': crypto, 'txid': txid}, modes=modes
    )


def get_unspent_outputs(crypto, address=None, addresses=None, services=None, **modes):
    if not services:
        services = get_optimal_services(crypto, 'unspent_outputs')

    kwargs = {'crypto': crypto}
    if addresses:
        kwargs['addresses'] = addresses
    if address:
        kwargs['address'] = address

    try:
        utxos = enforce_service_mode(
            services, UnspentOutputs, kwargs, modes=modes
        )
    except RevertToPrivateMode:
        # no services implement get_unspent_outputs_multi...
        modes['private'] = 1
        if modes.get('verbose'):
            print("Can't make with single API call. Retrying with private mode")
        utxos = enforce_service_mode(
            services, UnspentOutputs, kwargs, modes=modes
        )

    if modes.get('private'):
        # private mode returns items indexed by address, this only makes sense to do
        # for address balance, so remove it here
        just_utxos = []
        [just_utxos.extend(x) for x in utxos.values()]
        utxos = sorted(just_utxos, key=lambda tx: tx['output'])
        if modes.get('report_services', False):
            # private mode does not return services (its not practical),
            # an empty list is returned in its place to satisfy the API.
            return [], utxos

    return utxos


def get_historical_price(crypto, fiat, date):
    """
    Only one service is defined for geting historical price, so no fetching modes
    are needed.
    """
    return HistoricalPrice().action(crypto, fiat, date)


def push_tx(crypto, tx_hex, services=None, **modes):
    if not services:
        services = get_optimal_services(crypto, 'push_tx')
    return enforce_service_mode(
        services, PushTx, {'crypto': crypto, 'tx_hex': tx_hex}, modes=modes
    )


def get_block(crypto, block_number=None, block_hash=None, latest=False, services=None, **modes):
    if not services:
        services = get_optimal_services(crypto, 'get_block')
    kwargs = dict(crypto=crypto, block_number=block_number, block_hash=block_hash, latest=latest)
    return enforce_service_mode(
        services, GetBlock, kwargs, modes=modes
    )


def get_optimal_fee(crypto, tx_bytes, **modes):
    """
    Get the optimal fee based on how big the transaction is. Currently this
    is only provided for BTC. Other currencies will return $0.02 in satoshi.
    """
    try:
        services = get_optimal_services(crypto, 'get_optimal_fee')
    except NoServicesDefined:
        convert = get_current_price(crypto, 'usd')
        fee = int(0.02 / convert * 1e8)

        if modes.get('report_services'):
            return [None], fee
        else:
            return fee

    fee = enforce_service_mode(
        services, OptimalFee, dict(crypto=crypto, tx_bytes=tx_bytes), modes=modes
    )
    if modes.get('report_services'):
        return fee[0], int(fee[1])
    else:
        return int(fee)



def get_onchain_exchange_rates(deposit_crypto=None, withdraw_crypto=None, **modes):
    """
    Gets exchange rates for all defined on-chain exchange services.
    """
    from moneywagon.onchain_exchange import ALL_SERVICES

    rates = []
    for Service in ALL_SERVICES:
        srv = Service(verbose=modes.get('verbose', False))
        rates.extend(srv.onchain_exchange_rates())

    if deposit_crypto:
        rates = [x for x in rates if x['deposit_currency']['code'] == deposit_crypto.upper()]

    if withdraw_crypto:
        rates = [x for x in rates if x['withdraw_currency']['code'] == withdraw_crypto.upper()]

    if modes.get('best', False):
        return max(rates, key=lambda x: float(x['rate']))

    return rates


def generate_keypair(crypto, seed, password=None):
    """
    Generate a private key and publickey for any currency, given a seed.
    That seed can be random, or a brainwallet phrase.
    """
    if crypto in ['eth', 'etc']:
        raise CurrencyNotSupported("Ethereums not yet supported")

    pub_byte, priv_byte = get_magic_bytes(crypto)
    priv = sha256(seed)
    pub = privtopub(priv)

    priv_wif = encode_privkey(priv, 'wif_compressed', vbyte=priv_byte)
    if password:
        # pycrypto etc. must be installed or this will raise ImportError, hence inline import.
        from .bip38 import Bip38EncryptedPrivateKey
        priv_wif = str(Bip38EncryptedPrivateKey.encrypt(crypto, priv_wif, password))

    compressed_pub = encode_pubkey(pub, 'hex_compressed')
    ret = {
        'public': {
            'hex_uncompressed': pub,
            'hex': compressed_pub,
            'address': pubtoaddr(compressed_pub, pub_byte)
        },
        'private': {
            'wif': priv_wif
        }
    }
    if not password:
        # only these are valid when no bip38 password is supplied
        ret['private']['hex'] = encode_privkey(priv, 'hex_compressed', vbyte=priv_byte)
        ret['private']['hex_uncompressed'] = encode_privkey(priv, 'hex', vbyte=priv_byte)
        ret['private']['wif_uncompressed'] = encode_privkey(priv, 'wif', vbyte=priv_byte)

    return ret

def wif_to_address(crypto, wif):
    try:
        return privkey_to_address(wif, crypto_data[crypto]['address_version_byte'])
    except KeyError:
        raise CurrencyNotSupported("Currency not yet supported")

def sweep(crypto, private_key, to_address, fee=None, password=None, **modes):
    """
    Move all funds by private key to another address.
    """
    from moneywagon.tx import Transaction
    tx = Transaction(crypto, verbose=modes.get('verbose', False))
    tx.add_inputs(private_key=private_key, password=password, **modes)
    tx.change_address = to_address
    tx.fee(fee)

    return tx.push()


def get_explorer_url(crypto, address=None, txid=None, blocknum=None, blockhash=None):
    services = crypto_data[crypto]['services']['address_balance']
    urls = []
    context = {'crypto': crypto}
    if address:
        attr = "explorer_address_url"
        context['address'] = address
    elif txid:
        attr = "explorer_tx_url"
        context['txid'] = txid
    elif blocknum:
        attr = "explorer_blocknum_url"
        context['blocknum'] = blocknum
    elif blockhash:
        attr = "explorer_blockhash_url"
        context['blockhash'] = blockhash

    for service in services:
        template = getattr(service, attr)
        context['domain'] = service.domain
        context['protocol'] = service.protocol

        if hasattr(service, '_get_coin'):
            # used for when a service uses another name for a certain coin
            # other than the standard three letter currency code.
            context['coin'] = service._get_coin(crypto)

        if template:
            # render the explorer url temlate
            urls.append(template.format(**context))

    return urls


def guess_currency_from_address(address):
    """
    Given a crypto address, find which currency it likely belongs to.
    Raises an exception if it can't find a match. Raises exception if address
    is invalid.
    """
    if is_py2:
        fixer = lambda x: int(x.encode('hex'), 16)
    else:
        fixer = lambda x: x # does nothing

    first_byte = fixer(b58decode_check(address)[0])
    double_first_byte = fixer(b58decode_check(address)[:2])

    hits = []
    for currency, data in crypto_data.items():
        if hasattr(data, 'get'): # skip incomplete data listings
            version = data.get('address_version_byte', None)
            if version is not None and version in [double_first_byte, first_byte]:
                hits.append([currency, data['name']])

    if hits:
        return hits

    raise ValueError("Unknown Currency with first byte: %s" % first_byte)

def change_version_byte(address, new_version):
    """
    Convert the passed in address (or any base58 encoded string), and change the
    version byte to `new_version`.
    """
    payload = b58decode_check(address)[1:]
    return b58encode_check(chr(new_version) + payload)

class OptimalFee(AutoFallbackFetcher):
    def action(self, crypto, tx_bytes):
        crypto = crypto.lower()
        return self._try_services("get_optimal_fee", crypto, tx_bytes)

    def no_service_msg(self, crypto, tx_bytes):
        return "Could not get optimal fee for: %s" % crypto


class SingleTransaction(AutoFallbackFetcher):
    def action(self, crypto, txid):
        crypto = crypto.lower()
        return self._try_services("get_single_transaction", crypto, txid)

    @classmethod
    def strip_for_consensus(cls, result):
        return "%.8f %.8f" % (result['total_in'], result['total_out'])

    def no_service_msg(self, crypto, txid=None, txids=None):
        return "Could not get transaction info for: %s:%s" % (crypto, txid or ', '.join(txids))


class GetBlock(AutoFallbackFetcher):
    def action(self, crypto, block_number='', block_hash='', latest=False):
        if sum([type(block_number)==int, bool(block_hash), bool(latest)]) != 1:
            raise ValueError("Only one of `block_hash`, `latest`, or `block_number` allowed.")
        return self._try_services(
            'get_block', crypto, block_number=block_number, block_hash=block_hash, latest=latest
        )

    def no_service_msg(self, crypto, block_number=None, block_hash=None, latest=False):
        block = block_number or block_hash or ('latest' if latest else 'None')
        return "Could not get %s block: %s" % (
            crypto, block
        )

    @classmethod
    def strip_for_consensus(self, result):
        return "%s, %s, %s" % (
            result['hash'], result['block_number'], result['size']
        )


class HistoricalTransactions(AutoFallbackFetcher):
    def action(self, crypto, address=None, addresses=None):
        if addresses:
            method_name = "get_transactions_multi"
            kwargs = dict(addresses=addresses)

        if address:
            method_name = "get_transactions"
            kwargs = dict(address=address)

        txs = self._try_services(method_name, crypto, **kwargs)
        return sorted(txs, key=lambda tx: tx['date'], reverse=True)

    def no_service_msg(self, crypto, address=None, addresses=None):
        return "Could not get transactions for: %s:%s" % (crypto, address or ', '.join(addresses))

    @classmethod
    def strip_for_consensus(cls, results):
        stripped = []
        for result in results:
            result.sort(key=lambda x: x['date'])
            stripped.append(
                ", ".join(
                    ["[id: %s, amount: %s]" % (x['txid'], x['amount']) for x in result]
                )
            )
        return stripped


class UnspentOutputs(AutoFallbackFetcher):
    def action(self, crypto, address=None, addresses=None):
        if addresses:
            method_name = "get_unspent_outputs_multi"
            kwargs = dict(addresses=addresses)

        if address:
            method_name = "get_unspent_outputs"
            kwargs = dict(address=address)

        utxos = self._try_services(method_name, crypto=crypto, **kwargs)
        return sorted(utxos, key=lambda x: x['output'])

    def no_service_msg(self, crypto, address=None, addresses=None):
        return "Could not get unspent outputs for: %s:%s" % (crypto, address or ', '.join(addresses))

    @classmethod
    def strip_for_consensus(cls, results):
        stripped = []
        for result in results:
            result.sort(key=lambda x: x['output'])
            stripped.append(
                ", ".join(
                    ["[output: %s, value: %s]" % (x['output'], x['amount']) for x in result]
                )
            )
        return stripped


class CurrentPrice(AutoFallbackFetcher):
    def action(self, crypto, fiat, convert_to=None):
        if crypto.lower() == fiat.lower():
            return (1.0, 'math')

        ret = self._try_services('get_current_price', crypto=crypto, fiat=fiat)
        if convert_to:
            return ret / get_fiat_exchange_rate(from_fiat=fiat, to_fiat=convert_to)
        return ret


    def simplify_for_average(self, value):
        return value

    def no_service_msg(self, crypto, fiat):
        return "Can not find current price for %s->%s" % (crypto, fiat)


class AddressBalance(AutoFallbackFetcher):
    def action(self, crypto, address=None, addresses=None, confirmations=1):
        kwargs = dict(crypto=crypto, confirmations=confirmations)

        if address:
            method_name = "get_balance"
            kwargs['address'] = address

        if addresses:
            method_name = "get_balance_multi"
            kwargs['addresses'] = addresses

        results = self._try_services(method_name, **kwargs)

        if addresses and 'total_balance' not in results:
            results['total_balance'] = sum(results.values())

        return results

    def no_service_msg(self, crypto, address=None, addresses=None, confirmations=1):
        return "Could not get confirmed address balance for: %s" % crypto


class PushTx(AutoFallbackFetcher):
    def action(self, crypto, tx_hex):
        return self._try_services("push_tx", crypto=crypto, tx_hex=tx_hex)

    def no_service_msg(self, crypto, tx_hex):
        return "Could not push this %s transaction." % crypto


class HistoricalPrice(object):
    """
    This one doesn't inherit from AutoFallbackFetcher because there is only one
    historical price API service at the moment.
    """
    def __init__(self, responses=None, verbose=False):
        self.service = Quandl(responses, verbose=verbose)

    def action(self, crypto, fiat, at_time):
        crypto = crypto.lower()
        fiat = fiat.lower()

        if crypto != 'btc' and fiat != 'btc':
            # two external requests and some math is going to be needed.
            from_btc, source1, date1 = self.service.get_historical(crypto, 'btc', at_time)
            to_altcoin, source2, date2 = self.service.get_historical('btc', fiat, at_time)
            return (from_btc * to_altcoin), "%s x %s" % (source1, source2), date1
        else:
            return self.service.get_historical(crypto, fiat, at_time)

    @property
    def responses(self):
        return self.service.responses


def service_table(format='simple', authenticated=False):
    """
    Returns a string depicting all services currently installed.
    """
    if authenticated:
        all_services = ExchangeUniverse.get_authenticated_services()
    else:
        all_services = ALL_SERVICES

    if format == 'html':
        linkify = lambda x: "<a href='{0}' target='_blank'>{0}</a>".format(x)
    else:
        linkify = lambda x: x

    ret = []
    for service in sorted(all_services, key=lambda x: x.service_id):
        ret.append([
            service.service_id,
            service.__name__, linkify(service.api_homepage.format(
                domain=service.domain, protocol=service.protocol
            )),
            ", ".join(service.supported_cryptos or [])
        ])

    return tabulate(ret, headers=['ID', 'Name', 'URL', 'Supported Currencies'], tablefmt=format)

def wif_to_hex(wif):
    """
    Convert a WIF encded private key and return the raw hex encoded private key
    This function works for all bitcoin-API compatable coins.
    """
    return hexlify(b58decode_check(wif)[1:]).upper()

class ExchangeUniverse(object):
    def __init__(self, verbose=False, services=None, timeout=6):
        self._all_pairs = {}
        self.services = [
            x(verbose=verbose, timeout=timeout) if not isinstance(x, Service) else x
            for x in (services or ALL_SERVICES)
        ]
        self.verbose = verbose

    @classmethod
    def get_authenticated_services(self):
        return [x for x in ALL_SERVICES if x().api_key]

    def get_benchmarks(self):
        ret = {}
        for s in self.services:
            if not s.total_external_fetch_duration:
                continue # no call was made to this service, do not include in benchmark
            ret[s.name] = s.total_external_fetch_duration.total_seconds()
        return ret

    def fetch_pairs(self):
        if self._all_pairs:
            return

        for service in self.services:
            try:
                self._all_pairs[service.name] = service.get_pairs()
            except NotImplementedError:
                pass
            except Exception as exc:
                print("%s returned error: %s" % (service.__class__.__name__, exc))

    def find_pair(self, crypto="", fiat="", verbose=False):
        """
        This utility is used to find an exchange that supports a given exchange pair.
        """
        self.fetch_pairs()
        if not crypto and not fiat:
            raise Exception("Fiat or Crypto required")

        def is_matched(crypto, fiat, pair):
            if crypto and not fiat:
                return pair.startswith("%s-" % crypto)
            if crypto and fiat:
                return pair == "%s-%s" % (crypto, fiat)
            if not crypto:
                return pair.endswith("-%s" % fiat)

        matched_pairs = {}
        for Service, pairs in self._all_pairs.items():
            matched = [p for p in pairs if is_matched(crypto, fiat, p)]
            if matched:
                matched_pairs[Service] = matched

        return matched_pairs

    def all_cryptos(self):
        self.fetch_pairs()
        all_cryptos = set()
        for Service, pairs in self._all_pairs.items():
            for pair in pairs:
                crypto = pair.split("-")[0]
                all_cryptos.add(crypto)
        return sorted(all_cryptos)

    def most_supported(self, skip_supported=False):
        self.fetch_pairs()
        counts = []
        for crypto in self.all_cryptos():
            if skip_supported and crypto in crypto_data:
                continue
            matched = self.find_pair(crypto=crypto)
            count = sum(len(x) for x in matched.values())
            counts.append([crypto, count])

        return sorted(counts, key=lambda x: x[1], reverse=True)

def wif_to_address(crypto, wif):
    if is_py2:
        wif_byte = int(hexlify(b58decode_check(wif)[0]), 16)
    else:
        wif_byte = b58decode_check(wif)[0]

    if not wif_byte == crypto_data[crypto.lower()]['private_key_prefix']:
        msg = 'WIF encoded with wrong prefix byte. Are you sure this is a %s address?' % crypto.upper()
        raise Exception(msg)

    address_byte = crypto_data[crypto.lower()]['address_version_byte']
    return privkey_to_address(wif, address_byte)

def watch_mempool(crypto, callback=lambda tx: print(tx['txid']), verbose=False):
    if verbose:
        import logging
        logging.getLogger('socketIO-client').setLevel(logging.DEBUG)
        logging.basicConfig()

    services = get_optimal_services(crypto, 'address_balance')

    for service in services:
        socketio_url = service.socketio_url
        if not socketio_url:
            continue
        try:
            socketIO = SocketIO(socketio_url, verify=False,) # namespace=LoggingNamespace)
            socketIO.emit('subscribe', 'inv');
            socketIO.on('tx', callback)
            socketIO.wait()
        except KeyboardInterrupt:
            return
        except:
            print("failed")



from __future__ import print_function

import random
from crypto_data import crypto_data
from moneywagon import get_block, push_tx, get_single_transaction, watch_mempool
from .core import to_rawtx
from moneywagon.services import BitpayInsight, ChainSo, LocalBitcoinsChain, BlockDozer

class NetworkReplay(object):
    def __init__(self, source, destination, block_fetcher=None, verbose=False):
        """
        `block_fetcher` is a callable that will return a block.
        source is cryptocurrency network we get transactions from.
        destination is cryptocurrency network we will replay transactions to.
        """
        self.source = source
        self.destination = destination
        self.block_fetcher = block_fetcher
        self.verbose = verbose

        parent_currency, self.parent_fork_block = crypto_data[source.lower()].get('forked_from') or [None, 0]
        child_currency, self.child_fork_block = crypto_data[destination.lower()].get('forked_from') or [None, 0]

        if not parent_currency and not child_currency:
            raise Exception("Networks %s and %s are not forks of one another." % (
                source.upper(), destination.upper()
            ))

        if source.lower() == 'btc':
            self.pusher = BlockDozer(verbose=verbose)
            self.tx_fetcher = random.choice([BitpayInsight, ChainSo, LocalBitcoinsChain])(verbose=verbose)
        elif source.lower() == 'bch':
            self.pusher = random.choice(crypto_data['btc']['services']['push_tx'])(verbose=verbose)
            self.tx_fetcher = BlockDozer(verbose=verbose)
        else:
            raise Exception("Only BTC and BCH are currently supported")


    def replay_block(self, block_to_replay, limit=5):
        """
        Replay all transactions in parent currency to passed in "source" currency.
        Block_to_replay can either be an integer or a block object.
        """

        if block_to_replay == 'latest':
            if self.verbose:
                print("Getting latest %s block header" % source.upper())
            block = get_block(self.source, latest=True, verbose=self.verbose)
            if self.verbose:
                print("Latest %s block is #%s" % (self.source.upper(), block['block_number']))
        else:
            blocknum = block_to_replay if type(block_to_replay) == int else block_to_replay['block_number']
            if blocknum < self.parent_fork_block or blocknum < self.child_fork_block:
                raise Exception("Can't replay blocks mined before the fork")

        if type(block_to_replay) is not dict:
            if self.verbose:
                print("Getting %s block header #%s" % (self.source.upper(), block_to_replay))
            block = get_block(self.source, block_number=int(block_to_replay), verbose=self.verbose)
        else:
            block = block_to_replay

        if self.verbose:
            print("Using %s for pushing to %s" % (self.pusher.name, self.destination.upper()))
            print("Using %s for getting %s transactions" % (self.tx_fetcher.name, self.source.upper()))
            print("Finished getting block header,", len(block['txids']), "transactions in block, will replay", (limit or "all of them"))

        results = []
        enforced_limit = (limit or len(block['txids']))

        for i, txid in enumerate(block['txids'][:enforced_limit]):
            print("outside", txid)
            self._replay_tx(txid, i)


    def _replay_tx(self, txid, i=None):
        if self.block_fetcher:
            transaction = self.block_fetcher(currency=self.source, txid=txid)
        else:
            transaction = self.tx_fetcher.get_single_transaction(self.source, txid=txid)

        first_input = transaction['inputs'][0]
        if first_input.get('coinbase') or first_input.get("addresses") == "coinbase":
            return # no point trying to replay coinbases.

        raw_tx = to_rawtx(transaction)

        if self.verbose:
            print("got", txid)

        try:
            result = self.pusher.push_tx(self.destination, raw_tx)
            ret = ['success', result]
        except Exception as exc:
            ret = ['failure', str(exc)]

        if self.verbose:
            print("Result of push:", ret[0], ret[1])

        return ret

    def replay_mempool(self):
        watch_mempool(self.source, self._replay_tx)



import itertools
import collections
import binascii

from moneywagon.services import get_service
from moneywagon.crypto_data import crypto_data
from moneywagon import ALL_SERVICES

class CurrencySupport(object):
    support_categories = {
        'address': ['address_form', 'private_key_form'],
        'transaction': [ 'transaction_form', 'transaction_hash_algo', 'script_hash_algo'],
        'block': ['header_hash_algo'],
    }
    project_attributes = {
        'altcore': {
            'address_form': ('sha256-check', 'groestl-check'),
            'private_key_form': ('btc', ),
            'transaction_form': ('btc', ),
            'transaction_hash_algo': ('double-sha256', 'single-sha256'),
            'script_hash_algo': ('double-sha256', 'groestl'),
            'header_hash_algo': ('double-sha256', 'groestl')
        },
        'moneywagon': {
            'address_form': ('sha256-check', ),
            'private_key_form': ('btc', ),
            'transaction_form': ('btc', ),
            'transaction_hash_algo': ('double-sha256', ),
            'script_hash_algo': ('double-sha256', ),
            'header_hash_algo': ('double-sha256', )
        },
    }

    def __init__(self, verbose=False):
        self.verbose = verbose

    @property
    def sorted_crypto_data(self):
        return sorted(crypto_data.items(), key=lambda x: x[1]['name'])

    def is_all_supported(self, crypto_data_item, project, level):
        if level == 'full':
            fields_to_check = list(itertools.chain.from_iterable(self.support_categories.values()))
        else:
            fields_to_check = self.support_categories[level]

        for field, supported in self.project_attributes[project].items():
            value = crypto_data_item.get(field)
            if field not in fields_to_check:
                continue
            if value is not None and value not in supported:
                return False
        return True

    def supported_currencies(self, project='moneywagon', level="full"):
        """
        Returns a list of all currencies that are supported by the passed in project.
        and support level. Support level can be: "block", "transaction", "address"
        or "full".
        """
        ret = []
        if project == 'multiexplorer-wallet':
            for currency, data in self.sorted_crypto_data:
                if not data.get("bip44_coin_type"):
                    continue
                if len(data.get('services', {}).get("push_tx", [])) < 1:
                    continue
                if len(data.get('services', {}).get("historical_transactions", [])) < 1:
                    continue
                if len(data.get('services', {}).get("single_transaction", [])) < 1:
                    continue
                if len(data.get('services', {}).get("unspent_outputs", [])) < 1:
                    continue
                ret.append(currency)

            altcore_tx = self.supported_currencies('altcore', level="transaction")
            return [x for x in ret if x in altcore_tx]

        for symbol, data in self.sorted_crypto_data:
            if symbol == '': # template
                continue
            if self.is_all_supported(data, project, level):
                ret.append(symbol)
        return ret

    def not_supported_currencies(self, project='moneywagon', level="full"):
        """
        Returns a list of all currencies that are defined in moneywagon, by not
        supported by the passed in project and support level. Support level can
        be: "block", "transaction", "address" or "full".
        """
        supported = self.supported_currencies(project, level)
        ret = []
        for symbol, data in self.sorted_crypto_data:
            if symbol == '': # template
                continue
            if symbol not in supported:
                ret.append(symbol)
        return ret

    def altcore_data(self):
        """
        Returns the crypto_data for all currencies defined in moneywagon that also
        meet the minimum support for altcore. Data is keyed according to the
        bitcore specification.
        """
        ret = []
        for symbol in self.supported_currencies(project='altcore', level="address"):
            data = crypto_data[symbol]
            priv = data.get('private_key_prefix')
            pub = data.get('address_version_byte')
            hha = data.get('header_hash_algo')
            shb = data.get('script_hash_byte')

            supported = collections.OrderedDict()
            supported['name'] = data['name']
            supported['alias'] = symbol

            if pub is not None:
                supported['pubkeyhash'] = int(pub)
            if priv:
                supported['privatekey'] = priv
            supported['scripthash'] = shb if shb else 5
            if 'transaction_form' in data:
                supported['transactionForm'] = data['transaction_form']
            if 'private_key_form' in data:
                supported['privateKeyForm'] = data['private_key_form']
            #if 'message_magic' in data and data['message_magic']:
            #    supported['networkMagic'] = '0x%s' % binascii.hexlify(data['message_magic'])
            supported['port'] = data.get('port') or None
            if hha not in (None, 'double-sha256'):
                supported['headerHashAlgo'] = hha
            if data.get('script_hash_algo', 'double-sha256') not in (None, 'double-sha256'):
                supported['scriptHashAlgo'] = data['script_hash_algo']
            if data.get('transaction_hash_algo', 'double-sha256') not in (None, 'double-sha256'):
                supported['transactionHashAlgo'] = data['transaction_hash_algo']
            if data.get('seed_nodes'):
                supported['dnsSeeds'] = data['seed_nodes']

            ret.append(supported)

        return ret


def service_support(method=None, service=None, timeout=0.1, verbose=False):
    possible_args = {
        'get_current_price': ['btc', 'usd'],
        'get_balance': ['btc', '123445'],
        'get_orderbook': ['btc', 'usd'],
        'push_tx': ['btc', '23984729847298'],
        'make_order': ['btc', 'usd', 9999999999, 99999999, "sell"],
        'get_exchange_balance': ['btc'],
        'get_deposit_address': ['btc'],
        'initiate_withdraw': ['btc', 999999999999, '123456'],
    }
    matched = []

    def determine_support(s, method):
        try:
            getattr(s, method)(*possible_args[method])
        except NotImplementedError:
            if verbose:
                print("not implemented", s.name)
            return False
        except Exception as exc:
            if verbose:
                print ("implemented", s.name, exc, str(exc))
            return True
        return True

    if method:
        for Service in ALL_SERVICES:
            s = Service(timeout=timeout)
            if determine_support(s, method):
                matched.append(s.name)

    elif service:
        s = get_service(name=service)(timeout=timeout)
        for method in possible_args:
            if determine_support(s, method):
                matched.append(method)

    return matched



from datetime import datetime
from .services import *
from moneywagon.core import make_standard_halfing_eras

# instructions for getting version byte:
# https:/github.com/MichaelMure/WalletGenerator.net/wiki/How-to-add-a-new-currency#step-two-find-the-prefixes-for-the-address-format-of-your-currency

# also here: https://github.com/MichaelMure/WalletGenerator.net/blob/master/src/janin.currency.js#L78

crypto_data = {
    'btc': {
        'name': 'Bitcoin Legacy',
        'address_version_byte': 0, # base58Prefixes[PUBKEY_ADDRESS] in chainparams.cpp
        'message_magic': b"\xf9\xbe\xb4\xd9", # pchMessageStart in chainparams.cpp
        'bip44_coin_type': 0x80000000,
        'private_key_prefix': 128,  # base58Prefixes[SECRET_KEY] in chainparams.cpp
        'script_hash_byte': 0x05, # base58Prefixes[SCRIPT_ADDRESS] in chainparams.cpp
        'genesis_date': datetime(2009, 1, 3, 18, 15, 5),
        'header_hash_algo': 'double-sha256',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'github_link': "https://github.com/Bitcoin/bitcoin",
        'port': 8333,
        'seed_nodes': [
            "seed.b-pay.net",
            "seed.ob1.io"
            "seed.blockchain.info",
            "seed.bloq.com",
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 10,
            'full_cap': 21000000,
            'blocks_per_era': 210000,
            'reward_ends_at_block': 6930000
        },
        'services': {
            'current_price': {
                'usd': [
                    Bitstamp, GDAX, Wex, Gemini, Huobi, Bittrex, CexIO, YoBit,
                    Poloniex, Winkdex, ChainSo, Kraken, BitFinex, xBTCe, Vircurex,
                    HitBTC, LiveCoin, BitFlyer, ItBit, KuCoin, CCex, OKEX, BitZ, ZB,
                    EXX
                ],
                'cny': [BTER, BTCChina, Huobi, ViaBTC, xBTCe, BTC38, ChainSo],
                'rur': [Wex, LiveCoin], 'jpy': [BitFlyer, Kraken, xBTCe, Zaif, BitBank],
                'gbp': [Kraken, xBTCe],
                'eur': [Bitstamp, HitBTC, xBTCe, UseCryptos, LiveCoin, Kraken, ChainSo, ItBit],
                'cad': [Kraken], 'zar': [BitX], 'myr': [BitX], 'ngn': [BitX],
                'idr': [BitcoinIndonesia, BitX], 'mxn': [Bitso], 'bgn': [CryptoBG],
                'krw': [CoinOne, CoinNest], 'sgd': [ItBit], '*': [Cryptonator, Yunbi]
            },
            'address_balance': [
                BlockCypher, ChainSo,
                BitEasy, SmartBitAU, BlockExplorerCom, BlockChainInfo, Blockonomics,
                BitpayInsight, CoinPrism, BitGo, LocalBitcoinsChain, Bchain
            ],
            'historical_transactions': [
                BlockExplorerCom, BitGo, SmartBitAU, ChainSo, CoinPrism, BlockSeer,
                BitpayInsight, Blockonomics, LocalBitcoinsChain
            ],
            'single_transaction': [
                BitpayInsight, BlockCypher, BlockExplorerCom,
                ChainSo, CoinPrism, SmartBitAU, LocalBitcoinsChain
            ],
            'push_tx': [
                BlockChainInfo, BlockExplorerCom, ChainSo, CoinPrism,
                BitpayInsight, LocalBitcoinsChain
            ],
            'unspent_outputs': [
                BitpayInsight, BlockExplorerCom, SmartBitAU, BlockChainInfo, CoinPrism, ChainSo,
                BitGo, LocalBitcoinsChain
            ],
            'get_block': [
                BitpayInsight, ChainRadar, OKcoin, BlockExplorerCom, ChainSo,
                BitGo, LocalBitcoinsChain, BitFlyer
            ],
            "get_optimal_fee": [
                BitGo, BlockCypher, CoinTape, BitcoinFees21
            ]
        },
    },
    'ltc': {
        'name': 'Litecoin',
        'address_version_byte': 48,
        'message_magic': b"\xfb\xc0\xb6\xdb",
        'bip44_coin_type': 0x80000002,
        'private_key_prefix': 176,
        'genesis_date': datetime(2011, 10, 7),
        'header_hash_algo': 'scrypt',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': 9333,
        'seed_nodes': [
            'dnsseed.litecointools.com',
            'dnsseed.litecoinpool.org',
            'dnsseed.ltc.xurious.com',
            'dnsseed.koin-project.com',
            'dnsseed.weminemnc.com'
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 2.5,
            'full_cap': 84000000,
            'blocks_per_era': 840000
        },
        'services': {
            'current_price': {
                'usd': [
                    Wex, GDAX, Poloniex, Kraken, Cryptopia, ChainSo, OKEX,
                    xBTCe, YoBit, UseCryptos, HitBTC, LiveCoin, BitFinex, CCex,
                    EXX
                ],
                'cny': [Wex, Huobi, BTER, xBTCe, ViaBTC, ChainSo, OKcoin, BTC38, BTCChina],
                'rur': [YoBit, xBTCe], 'eur': [Kraken, HitBTC, Wex, GDAX, UseCryptos, xBTCe],
                'btc': [
                    GDAX, Wex, BTER, BleuTrade, HitBTC, Bittrex, Poloniex,
                    ChainSo, xBTCe, YoBit, Cryptopia, Kraken, NovaExchange, BitFinex,
                    BitcoinIndonesia, BTCChina, Liqui, LiveCoin, CryptoDao, KuCoin,
                    CCex, OKEX, BitZ, CoinEgg, ZB, BitBank, EXX
                ],
                'doge': [CCex], 'xmr': [Poloniex], 'jpy': [xBTCe], 'nzd': [Cryptopia],
                '*': [Cryptonator, Vircurex, YoBit], 'qc': [EXX]
            },
            'address_balance': [
                BlockCypher, Litecore, ChainSo, ProHashing, HolyTransaction, Bchain
            ],
            'historical_transactions': [
                ProHashing, ChainSo, Litecore
            ],
            'single_transaction': [
                BlockCypher, Litecore
            ],
            'push_tx': [
                ChainSo, Litecore
            ],
            'unspent_outputs': [
                Litecore, ChainSo
            ],
            'get_block': [
                ChainSo, OKcoin, ProHashing, Litecore, HolyTransaction
            ],
            "get_optimal_fee": [
                BlockCypher
            ]
        },
    },
    'ppc': {
        'name': 'Peercoin',
        'address_version_byte': 55,
        'message_magic': b"\xe6\xe8\xe9\xe5",
        'bip44_coin_type': 0x80000006,
        'private_key_prefix': 183,
        'genesis_date': datetime(2012, 8, 19, 18, 12, 4),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'transaction_form': 'ppc-timestamp',
        'github_link': "https://github.com/peercoin/peercoin",
        'port': 9901,
        'supply_data': {
            'method': 'standard',
            'minutes_per_block': 8.2
        },
        'services': {
            'current_price': {
                'usd': [Wex, xBTCe, ChainSo, LiveCoin], 'eur': [UseCryptos],
                'btc': [
                    Wex, xBTCe, BleuTrade, ChainSo, Bittrex, BTER, Poloniex, Vircurex,
                    YoBit, UseCryptos, BitZ
                ], 'doge': [BleuTrade], 'cny': [BTC38],
                '*': [Cryptonator]
            },
            'address_balance': [CryptoID, Mintr, HolyTransaction],
            'historical_transactions': [],
            'single_transaction': [Mintr],
            'push_tx': [MultiCoins],
            'unspent_outputs': [],
            'get_block': [Mintr, HolyTransaction]
        },
    },
    'doge': {
        'name': 'Dogecoin',
        'message_magic': b"\xc0\xc0\xc0\xc0",
        'address_version_byte': 30,
        'bip44_coin_type': 0x80000003,
        'private_key_prefix': 158,
        'script_hash_byte': 0x16,
        'genesis_date': datetime(2013, 12, 6, 10, 25, 40),
        'header_hash_algo': 'scrypt',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': 22556,
        'seed_nodes': [
            'seed.dogecoin.com',
            'seed.mophides.com',
            'seed.dglibrary.org',
            'seed.dogechain.info'
        ],
        'github_link': "https://github.com/dogecoin/dogecoin",
        'supply_data': {
            'method': 'per_era',
            'eras': [
                {'start': 1,      'end': 100000, 'reward': 500000}, # reward was random, average used
                {'start': 100001, 'end': 144999, 'reward': 250011}, # reward was random, average used
                {'start': 145000, 'end': 200000, 'reward': 250000},
                ] + make_standard_halfing_eras(
                    start=200000, interval=100000,
                    start_reward=125000, total_eras=4
                ) + [
                {'start': 600001, 'end': None,   'reward': 10000}
            ],
            'minutes_per_block': 1.0,
            'full_cap': None,
        },
        'services': {
            'current_price': {
                'usd': [CCex, HitBTC, LiveCoin], 'cny': [BTC38],
                'rur': [YoBit], 'ltc': [NovaExchange],
                'btc': [
                    Bittrex, Poloniex, BleuTrade, ChainSo, BTER, YoBit, HitBTC,
                    NovaExchange, Vircurex, BitcoinIndonesia, UseCryptos, LiveCoin,
                    BitZ, CoinEgg
                ],
                '*': [Cryptonator],
            },
            'address_balance': [
                BlockCypher, ChainSo, DogeChainInfo, ProHashing, HolyTransaction,
                Bchain
            ],
            'historical_transactions': [
                BlockCypher, ChainSo, ProHashing
            ],
            'single_transaction': [
                BlockCypher, ChainSo
            ],
            'push_tx': [
                ChainSo
            ],
            'unspent_outputs': [
                DogeChainInfo, ChainSo
            ],
            'get_block': [
                ChainSo, ProHashing, HolyTransaction
            ]
        },
    },
    'nxt': {
        'name': 'Nxt',
        'address_version_byte': None,
        'bip44_coin_type': 0x8000001d,
        'private_key_prefix': None,
        'genesis_date': datetime(2013, 10, 29),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'transaction_form': 'nxt',
        'port': None,
        'services': {
            'current_price': {
                'btc': [
                    Bittrex, BitcoinIndonesia, HitBTC, ChainSo, Poloniex,
                    LiveCoin, CoinEgg
                ],
                '*': [Cryptonator], 'cny': [BTC38], 'usd': [HitBTC],
            },
            'address_balance': [MyNXT, NXTPortal],
            'historical_transactions': [NXTPortal],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'xmy': {
        'name': 'MyriadCoin',
        'address_version_byte': 50,
        'message_magic': b"\xaf\x45\x76\xee",
        'bip44_coin_type': 0x8000005a,
        'private_key_prefix': 178,
        'script_hash_byte': 9,
        'genesis_date': datetime(2014, 2, 23),
        'transaction_form': 'cryptonote',
        'port': None,
        'github_link': "https://github.com/myriadteam/myriadcoin",
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 1000,
            'minutes_per_block': 0.5,
            'full_cap': 2000000000,
            'blocks_per_era': 967680
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Cryptopia], 'ltc': [Cryptopia],
                'doge': [Cryptopia], 'uno': [Cryptopia],
                '*': [Cryptonator]
            },
            'address_balance': [
                MYRCryptap, CryptapUS, BirdOnWheels, ProHashing
            ],
            'historical_transactions': [
                MYRCryptap, BirdOnWheels, ProHashing
            ],
            'single_transaction': [
                MYRCryptap, BirdOnWheels
            ],
            'push_tx': [
                MYRCryptap, BirdOnWheels
            ],
            'unspent_outputs': [
                MYRCryptap, BirdOnWheels
            ],
            'get_block': [
                MYRCryptap, BirdOnWheels, ProHashing
            ]
        },
    },
    'vtc': {
        'name': 'Vertcoin',
        'address_version_byte': 71,
        'message_magic': b"\xfa\xbf\xb5\xda",
        'bip44_coin_type': 0x8000001c,
        'private_key_prefix': 199,
        'genesis_date': datetime(2014, 1, 8),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 2.5,
            'full_cap': 84000000,
            'blocks_per_era': 840000
        },
        'services': {
            'current_price': {
                'btc': [Poloniex, Bittrex, ChainSo, BleuTrade, YoBit, CoinEgg],
                'doge': [BleuTrade],
                '*': [Cryptonator],
            },
            'address_balance': [VertcoinInfo, Bchain, VTConline],
            'historical_transactions': [VTConline],
            'single_transaction': [VertcoinInfo, VTConline],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [VertcoinInfo, VTConline]
        },
    },
    'ftc': {
        'name': 'Feathercoin',
        'address_version_byte': 14,
        'message_magic': b"\xfb\xc0\xb6\xdb",
        'bip44_coin_type': 0x80000008,
        'private_key_prefix': 0x8e,
        'genesis_date': datetime(2013, 4, 16),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'per_era',
            'minutes_per_block': 2.5,
            'eras': [
                {'start': 0,      'end': 204638, 'reward': 200},
                {'start': 204639, 'end': 2100000, 'reward': 80}
            ] + make_standard_halfing_eras(
                start=2100000, interval=2100000,
                start_reward=40
            ),
            'blocktime_adjustments': [
                [0, 2.632061418725984],
                [204639, 1]
            ],
            'full_cap': 336000000,
            'blocks_per_era': 2100000,
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, ChainSo, Cryptonator, Vircurex],
                'ltc': [Cryptopia], 'doge': [Cryptopia], 'uno': [Cryptopia],
                '*': [Cryptonator]
            },
            'address_balance': [
                ChainTips, FeathercoinCom2, Bchain
            ],
            'historical_transactions': [
                ChainTips, ProHashing
            ],
            'single_transaction': [
                ChainTips
            ],
            'push_tx': [
                ChainTips
            ],
            'unspent_outputs': [
                ChainTips
            ],
            'get_block': [
                ChainTips, ProHashing
            ]
        },
    },
    'dash': {
        'name': 'Dash',
        'address_version_byte': 76,
        "message_magic": b"\xbf\x0c\x6b\xbd",
        'bip44_coin_type': 0x80000005,
        'private_key_prefix': 204,
        'genesis_date': datetime(2014, 1, 19, 1, 40, 18),
        'header_hash_algo': 'x11',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': 9999,
        'seed_nodes': [
            'dnsseed.darkcoin.io',
            'dnsseed.dashdot.io',
            'dnsseed.masternode.io',
            'dnsseed.dashpay.io'
        ],
        'supply_data': {
            'method': 'per_era',
            'eras': [
                {'start': 1,   'end': 5000,  'reward': 500}, # "instamine"
                {'start': 5001, 'end': 20000,  'reward': 144},
                {'start': 20001, 'end': 60000, 'reward': 14}
            ] + make_standard_halfing_eras(
                start=60000, interval=210240,
                start_reward=5, halfing_func=lambda era, reward: reward - ((reward/14) * era)
            ),
            'additional_block_interval_adjustment_points': [
                 100, 500, 1000, 3000, 5000
            ],
            'minutes_per_block': 2.5,
            'full_cap': 18900000,
        },
        'services': {
            'current_price': {
                'usd': [
                    HitBTC, CexIO, Cryptopia, BitFinex, Kraken, LiveCoin, xBTCe, Liqui,
                    CCex, ZB
                ],
                'doge': [NovaExchange, Cryptopia, BleuTrade], 'eth': [Liqui, HitBTC],
                'rur': [YoBit], 'ltc': [NovaExchange, Cryptopia, CCex], 'gdp': [CexIO],
                'btc': [
                    Bittrex, Poloniex, Cryptopia, ChainSo, YoBit, CexIO, BleuTrade,
                    NovaExchange, Kraken, UseCryptos, Liqui, HitBTC, LiveCoin,
                    CryptoDao, BitFinex, CCex, BitZ, ZB
                ], 'cny': [xBTCe, BTC38], 'eur': [UseCryptos, Kraken],
                '*': [Cryptonator], 'uno': [Cryptopia], 'moon': [NovaExchange]
            },
            'address_balance': [
                CryptoID, ProHashing, MasterNodeIO, SiampmDashInsight, HolyTransaction,
                DashOrgInsight
            ],
            'historical_transactions': [
                ProHashing, DashOrgInsight, SiampmDashInsight, MasterNodeIO
            ],
            'single_transaction': [
                DashOrgInsight, SiampmDashInsight, MasterNodeIO, CryptoID
            ],
            'push_tx': [
                MasterNodeIO, DashOrgInsight, SiampmDashInsight
            ],
            'unspent_outputs': [
                MasterNodeIO, SiampmDashInsight, DashOrgInsight, CryptoID
            ],
            'get_block': [
                ProHashing, SiampmDashInsight, HolyTransaction, MasterNodeIO,
                DashOrgInsight
            ]
        },
    },
    'rdd': {
        'name': 'Reddcoin',
        'address_version_byte': 61,
        'bip44_coin_type': 0x80000004,
        'private_key_prefix': 0xbd,
        'genesis_date': datetime(2014, 1, 20),
        'proof_of_stake': True,
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'transaction_form': 'ppc-timestamp',
        'port': 45444,
        'last_pow_block': 260799,
        'seed_nodes': [
            'seed.reddcoin.com'
        ],
        'services': {
            'current_price': {
                'btc': [Bittrex, YoBit, BleuTrade, Cryptopia, NovaExchange],
                'ltc': [Cryptopia, NovaExchange], 'esp2': [NovaExchange],
                'doge': [Cryptopia, BleuTrade], 'uno': [Cryptopia], '*': [Cryptonator]
            },
            'address_balance': [ReddcoinCom, ProHashing],
            'historical_transactions': [ReddcoinCom, ProHashing],
            'single_transaction': [ReddcoinCom],
            'push_tx': [ReddcoinCom],
            'unspent_outputs': [ReddcoinCom],
            'get_block': [ReddcoinCom, ProHashing],
        },
    },

    'nmc': {
        'name': 'Namecoin',
        'address_version_byte': 52,
        'bip44_coin_type': 0x80000007,
        'private_key_prefix': 0x80,
        'genesis_date': datetime(2011, 4, 18),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Poloniex, BleuTrade, YoBit, xBTCe, Cryptopia, Vircurex, LiveCoin],
                'ltc': [Cryptopia], 'usd': [xBTCe],
                'doge': [Cryptopia, BleuTrade], 'uno': [Cryptopia], '*': [Cryptonator]
            },
            'address_balance': [WebBTC, Bchain]
        }
    },
    'aur': {
        'name': 'Auroracoin',
        'address_version_byte': 23,
        'bip44_coin_type': 0x80000055,
        'private_key_prefix': 0x97,
        'genesis_date': datetime(2014, 2, 2),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Bittrex, YoBit, Cryptopia], 'ltc': [Cryptopia],
                'doge': [Cryptopia], 'uno': [Cryptopia], '*': [Cryptonator]
            },
            'address_balance': [CryptoID, ProHashing],
            'historical_transactions': [ProHashing],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'emc': {
        'name': 'Emercoin',
        'address_version_byte': 33,
        'bip44_coin_type': None,
        'private_key_prefix': 128,
        'message_magic': b"\xe6\xe8\xe9\xe5",
        'genesis_date': datetime(2011, 11, 5),
        'github_link': "https://github.com/emercoin/emercoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [YoBit, HitBTC, Bittrex, xBTCe, Cryptopia, LiveCoin],
                'ltc': [Cryptopia], 'dash': [LiveCoin], 'eth': [LiveCoin],
                'usd': [xBTCe, LiveCoin], 'cny': [BTC38], 'rur': [LiveCoin],
                'xmr': [LiveCoin], 'doge': [Cryptopia], 'uno': [Cryptopia],
                '*': [Cryptonator]
            },
            'address_balance': [Mintr],
            'historical_transactions': [],
            'single_transaction': [Mintr],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [Mintr]
        },
    },

    'gsm': {
        'name': 'GSMcoin',
        'address_version_byte': 38,
        'bip44_coin_type': None,
        'private_key_prefix': 166,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/gsmcoin/GSMcoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {'btc': [YoBit]},
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },

    'erc': {
        'name': 'Europecoin',
        'address_version_byte': 33,
        'bip44_coin_type': None,
        'private_key_prefix': 161,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/EuropecoinEUORG/EuropecoinV2",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Bittrex],
                '*': [Cryptonator],
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },

    'tx': {
        'name': 'TransferCoin',
        'address_version_byte': 66,
        'bip44_coin_type': None,
        'private_key_prefix': 153,
        'message_magic': b"\xd1\x2e\x1e\xe6",
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/transferdev/Transfercoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Bittrex, YoBit, Cryptopia, LiveCoin], 'ltc': [Cryptopia],
                'doge': [Cryptopia], 'uno': [Cryptopia], '*': [Cryptonator]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'maid': {
        'name': 'MaidSafeCoin',
        'address_version_byte': 0,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Bittrex, Poloniex, Cryptopia, HitBTC, LiveCoin], 'ltc': [Cryptopia],
                'usd': [HitBTC],
                'doge': [Cryptopia], 'uno': [Cryptopia], '*': [Cryptonator]
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'tips': {
        'name': 'FedoraCoin',
        'address_version_byte': 33,
        'bip44_coin_type': None,
        'private_key_prefix': 161   ,
        'genesis_date': datetime(2013, 12, 21),
        'github_link': "https://github.com/fedoracoin/fedoracoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'ltc': [BTER, NovaExchange], 'moon': [NovaExchange],
                '*': [Cryptonator], 'piggy': [NovaExchange]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'karma': {
        'name': 'KarmaCoin',
        'address_version_byte': 45,
        'bip44_coin_type': None,
        'private_key_prefix': 173,
        'genesis_date': datetime(2014, 2, 4),
        'github_link': "https://github.com/karmacoin-team/karmacoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [YoBit],
                '*': [Cryptonator],
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'flap': {
        'name': 'FlappyCoin',
        'address_version_byte': 35,
        'bip44_coin_type': None,
        'private_key_prefix': 163,
        'genesis_date': datetime(2014, 2, 14),
        'github_link': "https://github.com/flappycoin-project/flappycoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {},
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'pot': {
        'name': 'PotCoin',
        'address_version_byte': 55,
        'bip44_coin_type': 0x80000051,
        'private_key_prefix': 183,
        'genesis_date': datetime(2014, 1, 21),
        'github_link': "https://github.com/potcoin/Potcoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 420,
            'minutes_per_block': 0.666666666,
            'full_cap': 420000000,
            'blocks_per_era': 210000
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Cryptopia, BleuTrade], 'ltc': [Cryptopia],
                'doge': [Cryptopia, BleuTrade], 'uno': [Cryptopia], '*': [Cryptonator],
            },
            'address_balance': [CryptoID, ProHashing],
            'historical_transactions': [ProHashing],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [ProHashing]
        },
    },
    'bqc': {
        'name': 'BBQcoin',
        'address_version_byte': 85,
        'bip44_coin_type': None,
        'private_key_prefix': 213,
        'genesis_date': datetime(2012, 7, 15),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                '*': [Cryptonator],
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'nvc': {
        'name': 'Novacoin',
        'address_version_byte': 8,
        'bip44_coin_type': 0x80000032,
        'private_key_prefix': 136,
        'genesis_date': datetime(2012, 10, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [YoBit, Cryptopia, BleuTrade, LiveCoin], 'ltc': [Cryptopia],
                'doge': [Cryptopia, BleuTrade], '*': [Cryptonator],
            },
            'address_balance': [CryptapUS, Bchain],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'uno': {
        'name': 'Unobtanium',
        'address_version_byte': 130,
        'bip44_coin_type': 0x8000005c,
        'private_key_prefix': 224,
        'genesis_date': datetime(2013, 10, 17),
        'header_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'full_cap': 250000,
            'blocks_per_era': 100000,
            'minutes_per_block': 3,
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, CCex, Cryptopia, BleuTrade], 'ltc': [Cryptopia],
                'usd': [CCex, Cryptopia], 'doge': [Cryptopia, BleuTrade],
                '*': [Cryptonator],
            },
            'address_balance': [UNOCryptap, CryptoID],
            'historical_transactions': [UNOCryptap, CryptoID],
            'single_transaction': [UNOCryptap],
            'push_tx': [UNOCryptap],
            'unspent_outputs': [UNOCryptap, CryptoID],
            'get_block': [UNOCryptap]
        },
    },
    'ric': {
        'name': 'Riecoin',
        'address_version_byte': 60,
        'bip44_coin_type': 0x8000008f,
        'private_key_prefix': 128,
        'genesis_date': datetime(2014, 2, 11, 0, 49, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 2.5,
            'blocks_per_era': 840000,
        },
        'services': {
            'current_price': {
                'btc': [Poloniex], 'cny': [BTC38],
                '*': [Cryptonator],
            },
            'address_balance': [RICCryptap, Bchain, CryptoID],
            'historical_transactions': [RICCryptap],
            'single_transaction': [RICCryptap, CryptoID],
            'push_tx': [RICCryptap],
            'unspent_outputs': [RICCryptap, CryptoID],
            'get_block': [RICCryptap]
        },
    },
    'xrp': {
        'name': 'Ripple',
        'genesis_date': datetime(2011, 3, 1),
        'bip44_coin_type': 0x80000090,
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [
                    Bittrex, Bitstamp, Poloniex, BitFinex, CoinEgg,
                    HitBTC, Bitso, BitcoinIndonesia, Kraken, ZB
                ],
                'cad': [Kraken], 'usd': [BitFinex, Bitstamp, Kraken, xBTCe, ZB],
                'eur': [Bitstamp, Kraken, xBTCe], 'jpy': [Kraken, BitBank], 'mxn': [Bitso],
                'krw': [CoinOne], 'cny': [BTC38], '*': [Cryptonator], 'qc': [ZB],
            },
        }
    },
    'thc': {
        'name': 'HempCoin',
        'address_version_byte': 40,
        'bip44_coin_type': None,
        'private_key_prefix': 168,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/hempcoin-project/THC",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.0,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex], '*': [Cryptonator],
            },
            'address_balance': [THCBlock, BlockExperts],
            'historical_transactions': [],
            'single_transaction': [THCBlock, BlockExperts],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [THCBlock, BlockExperts]
        },
    },
    'dope': {
        'name': 'Dopecoin',
        'address_version_byte': 8,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Bittrex, Cryptonator],
                '*': [Cryptonator],
            },
            'address_balance': [BlockExperts],
            'historical_transactions': [],
            'single_transaction': [BlockExperts],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [BlockExperts]
        },
    },
    'dime': {
        'name': 'Dimecoin',
        'address_version_byte': 15,
        'bip44_coin_type': None,
        'script_hash_byte': 9,
        'private_key_prefix': 143,
        'genesis_date': datetime(2014, 12, 23),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'seed_nodes': [
            'dime.mine-pool.net:11931',
            '209.126.65.116:11931',
            '122.10.88.44:11931',
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.066666667,
            'full_cap': 528000000,
            'blocks_per_era': 512000,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, LiveCoin], 'doge': [Cryptopia], 'rur': [LiveCoin],
                'ltc': [Cryptopia], 'uno': [Cryptopia], 'eur': [LiveCoin],
                '*': [Cryptonator],
            },
            'address_balance': [BlockExperts, CryptoID],
            'historical_transactions': [],
            'single_transaction': [BlockExperts, CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [BlockExperts]
        },
    },
    'xcp': {
        'name': 'CounterParty',
        'address_version_byte': 0,
        'bip44_coin_type': 0x80000009,
        'private_key_prefix': 128,
        'genesis_date': datetime(2014, 1, 2),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Bittrex, Poloniex, Cryptonator],
                '*': [Cryptonator],
            },
            'address_balance': [CoinDaddy1, CoinDaddy2, CounterPartyChain],
            'historical_transactions': [CoinDaddy1, CoinDaddy2],
            'single_transaction': [CoinDaddy1, CoinDaddy2],
            'push_tx': [CoinDaddy1, CoinDaddy2],
            'unspent_outputs': [CoinDaddy1],
            'get_block': [CoinDaddy1, CoinDaddy2]
        },
    },
    'eth': {
        'name': 'Ethereum',
        'address_form': "eth-0xhex",
        'private_key_form': "eth-pkhex",
        'transaction_form': 'eth',
        'bip44_coin_type': 0x8000003c,
        'genesis_date': datetime(2015, 7, 30),
        'port': None,
        'supply_data': {
            'instamine': 72009990.50
        },
        'services': {
            'current_price': {
                'rur': [YoBit, xBTCe, LiveCoin], 'cny': [ViaBTC, Yunbi, xBTCe],
                'usd': [
                    GDAX, Kraken, BitFinex, HitBTC, BitZ, OKEX, xBTCe, Liqui,
                    LiveCoin, KuCoin, ZB, EXX
                ],
                'jpy': [Kraken, xBTCe], 'cad': [Kraken],  'krw': [CoinOne, CoinNest],
                'eur': [xBTCe, Kraken, HitBTC], 'gbp': [Kraken], 'mxn': [Bitso],
                'btc': [
                    Poloniex, GDAX, BitFinex, Kraken, xBTCe, BleuTrade, Bittrex,
                    CexIO, HitBTC, BitcoinIndonesia, EtherChain, YoBit, Cryptopia,
                    Bitso, Liqui, LiveCoin, BitFlyer, KuCoin, CCex, OKEX, BitZ, ZB,
                    BitBank, EXX
                ],
                'ltc': [Cryptopia, xBTCe], 'doge': [Cryptopia, BleuTrade], 'uno': [Cryptopia],
                '*': [Cryptonator],
            },
            'address_balance': [Etherscan, EtherChain, ETCchain, EthPlorer],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'etc': {
        'name': 'Ethereum Classic',
        'address_form': "eth-0xhex",
        'private_key_form': "eth-pkhex",
        'transaction_form': 'eth',
        'bip44_coin_type': 0x8000003d,
        'genesis_date': datetime(2016, 7, 20),
        'port': None,
        'services': {
            'current_price': {
                'btc': [
                    Poloniex, BitFinex, HitBTC, Bittrex, CCex, CoinEgg,
                    Kraken, Cryptopia, YoBit, Yunbi, OKEX, BitZ, ZB, EXX
                ],
                'usd': [Kraken, BitFinex, OKEX, CCex, HitBTC, ZB, EXX], 'doge': [Cryptopia],
                'ltc': [CCex, Cryptopia], 'uno': [Cryptopia], 'krw': [CoinOne, CoinNest],
                '*': [Cryptonator], 'eth': [Kraken, HitBTC, OKEX, EXX], 'eur': [Kraken],
                'qc': [EXX], 'hsr': [EXX]
            },
            'address_balance': [ETCchain],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'xmr': {
        'name': 'Monero',
        'address_version_byte': None,
        'bip44_coin_type': 0x80000080,
        'private_key_prefix': None,
        'genesis_date': datetime(2014, 4, 18),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [
                    Poloniex, Bittrex, BitFinex, HitBTC,
                    Kraken, Cryptopia, LiveCoin
                ],
                'usd': [BitFinex, HitBTC, Kraken, Cryptopia, LiveCoin],
                'ltc': [Cryptopia], 'doge': [Cryptopia], 'uno': [Cryptopia],
                'nzd': [Cryptopia], 'eur': [Kraken], 'eth': [HitBTC],
                '*': [Cryptonator],
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        }
    },
    'blk': {
        'name': 'Blackcoin',
        'message_magic': b"\x70\x35\x22\x05",
        'address_version_byte': 25,
        'bip44_coin_type': 0x8000000a,
        'private_key_prefix': 153,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Poloniex, Cryptopia, BitZ, UseCryptos, BleuTrade, LiveCoin, CoinEgg],
                'ltc': [Cryptopia], 'doge': [Cryptopia, BleuTrade], 'uno': [Cryptopia],
                '*': [Cryptonator], 'cny': [BTC38],
            },
            'address_balance': [
                CryptoID, HolyTransaction, Bchain
            ],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'xpm': {
        'name': 'Primecoin',
        'address_version_byte': 23,
        'bip44_coin_type': 0x80000018,
        'private_key_prefix': 151,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'btc': [Poloniex, BitZ, Cryptopia, Vircurex, BleuTrade, CoinEgg],
                'ltc': [Cryptopia], 'doge': [Cryptopia, BleuTrade],
                'uno': [Cryptopia], 'cny': [BTC38], '*': [Cryptonator],
            },
            'address_balance': [
                Bchain
            ],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'pivx': {
        'name': 'PivX',
        'address_version_byte': 30,
        'bip44_coin_type': 0x80000077,
        'private_key_prefix': 212,
        'script_hash_byte': 0xD,
        'message_magic': b"\x90\xc4\xfd\xe9",
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': 51472,
        'seed_nodes': [
            'pivx.seed.fuzzbawls.pw',
            'pivx.seed2.fuzzbawls.pw',
            'coin-server.com',
            's3v3nh4cks.ddns.net'
        ],
        'services': {
            'current_price': {
                'btc': [Bittrex, Cryptopia, YoBit, LiveCoin],
                'ltc': [Cryptopia], 'doge': [Cryptopia], 'uno': [Cryptopia],
                '*': [Cryptonator], 'eth': [LiveCoin], 'eur': [LiveCoin],
                'rur': [LiveCoin], 'xmr': [LiveCoin], 'usd': [Cryptopia, LiveCoin],
            },
            'address_balance': [PressTab, CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'zec': {
        'name': 'ZCash',
        'address_version_byte': 7352,
        'bip44_coin_type': 0x80000085,
        'private_key_prefix': 128,
        'script_hash_byte': 0x1cbd,
        'genesis_date': datetime(2016, 10, 28, 7, 56),
        'message_magic': b"\x24\xe9\x27\x64",
        'github_link': "https://github.com/zcash/zcash",
        'header_hash_algo': 'double-sha256',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': 8233,
        'services': {
            'current_price': {
                'btc': [Bittrex, BitFinex, HitBTC, YoBit, Kraken, BitZ],
                'usd': [BitFinex, Kraken, HitBTC], 'bch': [CoinEx],
                'eth': [HitBTC], 'eur': [Kraken], 'cny': [ViaBTC], '*': [Cryptonator],
            },
            'address_balance': [ZChain, MercerWeiss],
            'historical_transactions': [MercerWeiss],
            'single_transaction': [MercerWeiss],
            'push_tx': [MercerWeiss],
            'unspent_outputs': [MercerWeiss],
            'get_block': [MercerWeiss]
        },
    },
    'grs': {
        'name': 'Groestlcoin',
        'address_version_byte': 36,
        'bip44_coin_type': 0x80000011,
        'private_key_prefix': 128,
        'script_hash_byte': 0x05,
        'message_magic': b"\xF9\xBE\xB4\xD4",
        'genesis_date': datetime(2014, 3, 20, 19, 13, 49),
        'header_hash_algo': 'groestl',
        'transaction_hash_algo': 'single-sha256',
        'script_hash_algo': 'groestl',
        'address_form': 'groestl-check',
        'github_link': 'https://github.com/Groestlcoin/groestlcoin',
        'port': 1331,
        'seed_nodes': [
            'groestlcoin.org',
            'jswallet.groestlcoin.org',
            'electrum1.groestlcoin.org',
            'electrum2.groestlcoin.org'
        ],
        'services': {
            'current_price': {
                'btc': [Bittrex],
                '*': [Cryptonator],
            },
            'address_balance': [CryptoID, Groestlsight],
            'historical_transactions': [Groestlsight],
            'single_transaction': [CryptoID, Groestlsight],
            'push_tx': [Groestlsight],
            'unspent_outputs': [CryptoID, Groestlsight],
            'get_block': [Groestlsight]
        },
    },
    'sys': {
        'name': 'Syscoin 2.1',
        'address_version_byte': 0,
        'bip44_coin_type': 0x80000039,
        'private_key_prefix': 128,
        'script_hash_byte': 5,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': 'double-sha256',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'github_link': "https://github.com/syscoin/syscoin2",
        'services': {
            'current_price': {
                'btc': [Bittrex, YoBit, BTC38, LiveCoin],
                '*': [Cryptonator],
            },
            'address_balance': [ProHashing, CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'bun': {
        'name': 'BunnyCoin',
        'address_version_byte': 26,
        'bip44_coin_type': None,
        'private_key_prefix': 154,
        'script_hash_byte': 22,
        'genesis_date': datetime(2014, 4, 25, 16, 0),
        'header_hash_algo': 'scrypt',
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'services': {
            'current_price': {
                'ltc': [Cryptopia, NovaExchange], 'moon': [NovaExchange],
                'doge': [Cryptopia, NovaExchange], 'uno': [Cryptopia],
            },
            'address_balance': [ProHashing, CryptoChat],
            'historical_transactions': [CryptoChat],
            'single_transaction': [CryptoChat],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [CryptoChat]
        },
    },
    'bvc': {
        'name': 'BeaverCoin',
        'address_version_byte': 25,
        'bip44_coin_type': None,
        'private_key_prefix': 176,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 8,
            'minutes_per_block': 1,
            'full_cap': 3360000,
            'blocks_per_era': 210000,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Cryptopia], 'ltc': [Cryptopia], 'doge': [Cryptopia],
                'uno': [Cryptopia],
            },
            'address_balance': [ProHashing, BeavercoinBlockchain],
            'historical_transactions': [BeavercoinBlockchain],
            'single_transaction': [BeavercoinBlockchain],
            'push_tx': [BeavercoinBlockchain],
            'unspent_outputs': [BeavercoinBlockchain],
            'get_block': [BeavercoinBlockchain]
        },
    },
    'cat': {
        'name': 'Catcoin',
        'address_version_byte': 21,
        'bip44_coin_type': None,
        'private_key_prefix': 149,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 10,
            'full_cap': 21000000,
            'blocks_per_era': 210000,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {},
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'lemon': {
        'name': 'LemonCoin',
        'address_version_byte': 48,
        'bip44_coin_type': None,
        'private_key_prefix': 176,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 20,
            'minutes_per_block': 1,
            'full_cap': 31000000,
            'blocks_per_era': 525000,
            'reward_ends_at_block': None,
            'ico': 10000000,
            'ico_burn': 8960948,
        },
        'services': {
            'current_price': {
                'btc': [Cryptopia], 'ltc': [Cryptopia], 'doge': [Cryptopia],
                'uno': [Cryptopia],
            },
            'address_balance': [LemoncoinOfficial],
            'historical_transactions': [LemoncoinOfficial],
            'single_transaction': [LemoncoinOfficial],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [LemoncoinOfficial]
        },
    },
    'geert': {
        'name': 'Geertcoin',
        'address_version_byte': 38,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'message_magic': None,
        'genesis_date': datetime(2017, 2, 12, 6, 25, 23),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 200,
            'minutes_per_block': 2.5,
            'full_cap': None,
            'blocks_per_era': 24000,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Cryptopia], 'ltc': [Cryptopia], 'doge': [Cryptopia]
            },
            'address_balance': [GeertcoinExplorer],
            'historical_transactions': [GeertcoinExplorer],
            'single_transaction': [GeertcoinExplorer],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [GeertcoinExplorer]
        },
    },
    'ulm': {
        'name': 'UnlimitedCoin',
        'address_version_byte': 68,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'message_magic': None,
        'genesis_date': datetime(2017, 5, 19, 17, 55, 39),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {'btc': [NovaExchange]},
            'address_balance': [UnlimitedCoinOfficial],
            'historical_transactions': [UnlimitedCoinOfficial],
            'single_transaction': [UnlimitedCoinOfficial],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [UnlimitedCoinOfficial]
        },
    },
    'mars': {
        'name': 'MarsCoin',
        'address_version_byte': 50,
        'bip44_coin_type': 0x8000006b,
        'private_key_prefix': 178,
        'script_hash_byte': 5,
        'message_magic': None,
        'genesis_date': datetime(2014, 1, 1, 15, 37, 7),
        'header_hash_algo': 'scrypt',
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 2.05,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {'btc': [NovaExchange], 'ltc': [NovaExchange]},
            'address_balance': [MarscoinOfficial],
            'historical_transactions': [MarscoinOfficial],
            'single_transaction': [MarscoinOfficial],
            'push_tx': [MarscoinOfficial],
            'unspent_outputs': [MarscoinOfficial],
            'get_block': [MarscoinOfficial]
        },
    },
    'moon': {
        'name': 'Mooncoin',
        'address_version_byte': 3,
        'bip44_coin_type': None,
        'private_key_prefix': 131,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.5,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [CCex, NovaExchange, BleuTrade], 'usd': [CCex],
                'ltc': [CCex, NovaExchange, BleuTrade], 'doge': [CCex]
            },
            'address_balance': [ProHashing, Bchain],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'rby': {
        'name': 'Rubycoin',
        'address_version_byte': 60,
        'bip44_coin_type': 0x80000010,
        'private_key_prefix': 188,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.5,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Cryptopia], 'ltc': [Cryptopia],
                'doge': [Cryptopia], 'uno': [Cryptopia]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'pnd': {
        'name': 'PandaCoin',
        'address_version_byte': 55,
        'bip44_coin_type': 0x80000025,
        'private_key_prefix': 183,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.0,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit], 'ltc': [Cryptopia],
                'doge': [Cryptopia], 'uno': [Cryptopia]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'trc': {
        'name': 'TerraCoin',
        'address_version_byte': 0,
        'bip44_coin_type': 0x80000053,
        'private_key_prefix': 128,
        'message_magic': b"\x42\xba\xbe\x56",
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/clockuniverse/terracoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 20,
            'minutes_per_block': 2,
            'full_cap': None,
            'blocks_per_era': 1050000,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [CCex, Vircurex, Cryptopia], 'usd': [CCex, Vircurex],
                'ltc': [CCex, Cryptopia, NovaExchange],
                'doge': [CCex, Cryptopia, NovaExchange]
            },
            'address_balance': [TerracoinIO, Bchain],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': []
        },
    },
    'dgc': {
        'name': 'DigitalCoin',
        'address_version_byte': 30,
        'bip44_coin_type': 0x80000012,
        'private_key_prefix': 158,
        'message_magic': b"\xfb\xc0\xb6\xdb",
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/DGCDev/digitalcoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 0.666666666,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [BleuTrade, NovaExchange, Cryptopia, Vircurex],
                'usd': [Vircurex], 'eur': [Vircurex], 'cny': [BTC38],
                'ltc': [Cryptopia, NovaExchange, Vircurex],
                'doge': [Vircurex, BleuTrade, Cryptopia, NovaExchange]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'qrk': {
        'name': 'Quark',
        'address_version_byte': 58,
        'bip44_coin_type': None,
        'private_key_prefix': 186,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [NovaExchange, Vircurex, Cryptopia, BTC38],
                'ltc': [Vircurex, NovaExchange, Cryptopia], 'eur': [Vircurex],
                'usd': [Vircurex], 'doge': [Cryptopia, NovaExchange],
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': []
        },
    },
    'tse': {
        'name': 'Tattoocoin',
        'address_version_byte': 65,
        'bip44_coin_type': None,
        'private_key_prefix': 193,
        'message_magic': None,
        'genesis_date': datetime(2016, 12, 29, 21, 53, 10),
        'github_link': 'https://github.com/CryptoCoderz/Tattoocoin-TSE',
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, CCex, NovaExchange, Cryptopia],
                'ltc': [CCex, NovaExchange, Cryptopia], 'usd': [CCex],
                'doge': [CCex, Cryptopia],
            },
            'address_balance': [CryptoChat],
            'historical_transactions': [CryptoChat],
            'single_transaction': [CryptoChat],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [CryptoChat]
        },
    },
    'xvg': {
        'name': 'Verge',
        'address_version_byte': 30,
        'bip44_coin_type': 0x8000004d,
        'private_key_prefix': 158,
        'message_magic': None,
        'genesis_date': datetime(2014, 10, 9, 18, 22, 44),
        'github_link': "https://github.com/vergecurrency/VERGE",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 0.5,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, CCex, Bittrex, Cryptopia, NovaExchange],
                'ltc': [CCex, Cryptopia, NovaExchange], 'usd': [CCex],
                'doge': [CCex, NovaExchange, Cryptopia]
            },
            'address_balance': [VergeCurrencyInfo, ProHashing],
            'historical_transactions': [VergeCurrencyInfo],
            'single_transaction': [VergeCurrencyInfo],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [VergeCurrencyInfo]
        },
    },
    '1337': {
        'name': '1337',
        'address_version_byte': 48,
        'bip44_coin_type': None,
        'private_key_prefix': 176,
        'script_hash_byte': 28,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': 'https://github.com/Velvet78/1337',
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, Cryptopia, NovaExchange],
                'ltc': [Cryptopia, NovaExchange], 'moon': [NovaExchange],
                'doge': [Cryptopia, NovaExchange],
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [],
            "get_optimal_fee": []
        },
    },
    'cj': {
        'name': 'CryptoJacks',
        'address_version_byte': 28,
        'bip44_coin_type': None,
        'private_key_prefix': 156,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/cryptojacks/CryptoJacks",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [CCex, NovaExchange, Cryptopia, YoBit],
                'ltc': [CCex, NovaExchange, Cryptopia], 'usd': [CCex],
                'doge': [Cryptopia, NovaExchange]
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            "get_optimal_fee": []
        },
    },
    'fjc': {
        'name': 'FujiCoin',
        'address_version_byte': 36,
        'bip44_coin_type': 0x8000004b,
        'private_key_prefix': 164,
        'script_hash_byte': 16,
        'message_magic': b"\x66\x75\x6a\x69",
        'genesis_date': datetime(2014, 6, 27, 23, 0),
        'github_link': "https://github.com/fujicoin/fujicoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': 3777,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, CCex, NovaExchange, Cryptopia, BleuTrade],
                'ltc': [CCex, Cryptopia, NovaExchange], 'usd': [CCex],
                'doge': [CCex, NovaExchange, Cryptopia]
            },
            'address_balance': [FujiInsght],
            'historical_transactions': [FujiInsght],
            'single_transaction': [FujiInsght],
            'push_tx': [FujiInsght],
            'unspent_outputs': [FujiInsght],
            'get_block': [FujiInsght],
            "get_optimal_fee": []
        },
    },
    'note': {
        'name': 'DNotes',
        'address_version_byte': 31,
        'bip44_coin_type': None,
        'private_key_prefix': 159,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/DNotesCoin/DNotes/",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, CCex, Poloniex, NovaExchange, Cryptopia],
                'ltc': [CCex, Cryptopia, NovaExchange], 'usd': [CCex],
                'doge': [CCex, NovaExchange, Cryptopia]
            },
            'address_balance': [CryptoID, ProHashing],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [],
            "get_optimal_fee": []
        },
    },
    'song': {
        'name': 'SongCoin',
        'address_version_byte': 63,
        'bip44_coin_type': None,
        'private_key_prefix': 191,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/Songcoin/Songcoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 2,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, CCex, NovaExchange, Cryptopia],
                'ltc': [CCex, Cryptopia], 'usd': [CCex],
                'doge': [CCex, NovaExchange, Cryptopia]
            },
            'address_balance': [ProHashing],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'ptc': {
        'name': 'Pesetacoin',
        'address_version_byte': 47,
        'bip44_coin_type': 0x8000006d,
        'private_key_prefix': 175,
        'message_magic': b"\xc0\xc0\xc0\xc0",
        'genesis_date': datetime(2013, 12, 31, 22, 23, 8),
        'github_link': "https://github.com/pesetachain/pesetacoin-0.9",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'per_era',
            'eras': [
                {'start': 0,      'end': 525600, 'reward': 166.386},
                {'start': 525601, 'end': 550000, 'reward': 83.193},
                {'start': 550001, 'end': 1051200, 'reward': 50},
                {'start': 1051201, 'end': 1576800, 'reward': 25},
                {'start': 1576801, 'end': 2628000, 'reward': 10},
                {'start': 2628001, 'end': 8409600, 'reward': 5},
                {'start': 8409601, 'end': None, 'reward': 0},
            ],
            'full_cap': 166386000,
            'reward_ends_at_block': 8409601
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Cryptopia, NovaExchange], 'ltc': [Cryptopia],
                'uno': [Cryptopia], 'doge': [Cryptopia]
            },
            'address_balance': [PesetacoinInfo, CryptoID, PresetacoinInsight],
            'historical_transactions': [PesetacoinInfo, PresetacoinInsight],
            'single_transaction': [PesetacoinInfo, CryptoID, PresetacoinInsight],
            'push_tx': [PresetacoinInsight],
            'unspent_outputs': [PesetacoinInfo, CryptoID, PresetacoinInsight],
            'get_block': [PesetacoinInfo],
            'get_optimal_fee': []
        },
    },
    'tes': {
        'name': 'TeslaCoin',
        'address_version_byte': 11,
        'bip44_coin_type': None,
        'private_key_prefix': 139,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/TeslacoinFoundation/Teslacoin",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [YoBit, Cryptopia, NovaExchange], 'ltc': [Cryptopia, NovaExchange],
                'uno': [Cryptopia], 'doge': [Cryptopia, NovaExchange]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'xvc': {
        'name': 'Vcash',
        'address_version_byte': 71,
        'bip44_coin_type': None,
        'private_key_prefix': 199,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/openvcash/vcash",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Poloniex]
            },
            'address_balance': [VChainInfo],
            'historical_transactions': [],
            'single_transaction': [VChainInfo],
            'push_tx': [],
            'unspent_outputs': [VChainInfo],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'meow': {
        'name': 'KittehCoin',
        'address_version_byte': 45,
        'bip44_coin_type': None,
        'private_key_prefix': 173,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/kittehcoin/kittehcoin",
        'header_hash_algo': 'scrypt',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.0,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'ltc': [Cryptopia, NovaExchange], 'doge': [Cryptopia]
            },
            'address_balance': [ProHashing, CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'huc': {
        'name': 'HunterCoin',
        'address_version_byte': 40,
        'bip44_coin_type': None,
        'private_key_prefix': 168,
        'message_magic': b"\xf9\xbe\xb4\xfe",
        'genesis_date': datetime(1, 1, 1),
        'github_link': "https://github.com/chronokings/huntercore",
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 1.0,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Poloniex], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'bcc': {
        'name': 'BitConnect',
        'address_version_byte': 18,
        'bip44_coin_type': None,
        'private_key_prefix': 146,
        'script_hash_byte': 85,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [NovaExchange, LiveCoin], 'ltc': [], 'usd': [], 'doge': [NovaExchange]
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'dmd': {
        'name': 'Diamond',
        'address_version_byte': 90,
        'bip44_coin_type': None,
        'private_key_prefix': 218,
        'script_hash_byte': 8,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': 'https://github.com/DMDcoin/Diamond',
        'header_hash_algo': "groestl",
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [CryptoID],
            'historical_transactions': [],
            'single_transaction': [CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'nlc2': {
        'name': 'NoLimitCoin',
        'address_version_byte': 53,
        'bip44_coin_type': None,
        'private_key_prefix': 181,
        'script_hash_byte': 92,
        'message_magic': None,
        'genesis_date': datetime(2016, 5, 28, 20, 15),
        'github_link': 'https://github.com/NoLimitCoin/NoLimitCoin',
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': 2,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [NoLimitCoinIquidus],
            'historical_transactions': [],
            'single_transaction': [NoLimitCoinIquidus],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [NoLimitCoinIquidus],
            'get_optimal_fee': []
        },
    },
    'cloak': {
        'name': 'CloakCoin',
        'address_version_byte': 27,
        'bip44_coin_type': None,
        'private_key_prefix': 155,
        'script_hash_byte': 85,
        'message_magic': None,
        'genesis_date': datetime(2014, 5, 31, 11, 52, 35),
        "github_link": "https://github.com/cashmen/CloakCoinRelaunch",
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [LiveCoin, Bittrex, Cryptopia, YoBit, CryptoDao], 'ltc': [Cryptopia],
                'usd': [LiveCoin], 'doge': [Cryptopia]
            },
            'address_balance': [CloakCoinIquidus, CryptoID],
            'historical_transactions': [],
            'single_transaction': [CloakCoinIquidus, CryptoID],
            'push_tx': [],
            'unspent_outputs': [CryptoID],
            'get_block': [CloakCoinIquidus],
            'get_optimal_fee': []
        },
    },
    'mgc': {
        'name': 'MergeCoin',
        'address_version_byte': 50,
        'bip44_coin_type': 0x8000007c,
        'private_key_prefix': 178,
        'script_hash_byte': 5,
        'message_magic': b"\xc8\xe1\xd5\xec",
        'genesis_date': datetime(2017, 3, 23, 9, 23, 36),
        "github_link": "https://github.com/mergecoin-project/Mergecoin-master",
        'header_hash_algo': None,
        "transaction_form": "ppc-timestamp",
        'port': None,
        'seeds': [
            "47.89.43.73", "43.241.232.45", "47.89.178.249", "120.55.82.131",
            "121.41.19.30", "120.26.231.61", "47.94.89.212", "mergechain.com",
            "mergecoin.com"
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [CryptoDao], 'ltc': [], 'usd': [],
                'doge': [], 'cny': [BTC38]
            },
            'address_balance': [MergeCoinInsight],
            'historical_transactions': [MergeCoinInsight],
            'single_transaction': [MergeCoinInsight],
            'push_tx': [MergeCoinInsight],
            'unspent_outputs': [MergeCoinInsight],
            'get_block': [MergeCoinInsight],
            'get_optimal_fee': []
        },
    },
    'part': {
        'name': 'Particl',
        'address_version_byte': 56,
        'bip44_coin_type': 0x8000002c,
        'private_key_prefix': 108,
        'script_hash_byte': 60,
        'message_magic': b"\xfb\xf2\xef\xb4",
        'genesis_date': datetime(2017, 7, 17, 13, 0),
        'github_link': 'https://github.com/particl/particl-core',
        'header_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, BitZ], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [CryptoID, ParticlInsight],
            'historical_transactions': [ParticlInsight],
            'single_transaction': [CryptoID, ParticlInsight],
            'push_tx': [ParticlInsight],
            'unspent_outputs': [CryptoID, ParticlInsight],
            'get_block': [ParticlInsight],
            'get_optimal_fee': []
        },
    },
    'bch': {
        'name': 'Bitcoin Cash',
        'forked_from': ('btc', 478559),
        'address_version_byte': 0,
        'message_magic': b"\xf9\xbe\xb4\xd9",
        'bip44_coin_type': 0x80000091,
        'private_key_prefix': 128,
        'script_hash_byte': 0x05,
        'genesis_date': datetime(2009, 1, 3, 18, 15, 5),
        'transaction_form': 'btc-cash-anti-replay',
        'header_hash_algo': 'double-sha256',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'github_link': "https://github.com/Bitcoin-ABC/bitcoin-abc",
        'port': 8333,
        'seed_nodes': [
            "seed-abc.bitcoinforks.org",
            "seed.bitcoinabc.org",
            "btccash-seeder.bitcoinunlimited.info",
            "seed.bitprim.org",
            "seed.deadalnix.me",
            "seeder.criptolayer.net"
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': 50,
            'minutes_per_block': 10,
            'full_cap': 21000000,
            'blocks_per_era': 210000,
            'reward_ends_at_block': 6930000
        },
        'services': {
            'current_price': {
                'btc': [
                    Bitstamp, Kraken, Bittrex, HitBTC, OKEX, BitZ,
                    CoinEgg, ZB, BitBank, EXX
                ],
                'eur': [Kraken], 'usd': [HitBTC, Bitstamp, Kraken, OKEX, ZB], 'jpy': [BitBank],
                'eth': [HitBTC], 'krw': [CoinNest], '*': [Cryptonator]
            },
            'address_balance': [
                BitpayInsightBCH, TrezorBCH, BlockExplorerCash, BlockDozer,
                BitcoinComCashExplorer
            ],
            'historical_transactions': [
                BitcoinComCashExplorer, BitpayInsightBCH, TrezorBCH, BlockExplorerCash,
                BlockDozer
            ],
            'single_transaction': [
                BlockDozer, BitcoinComCashExplorer, BitpayInsightBCH, TrezorBCH,
                BlockExplorerCash
            ],
            'push_tx': [
                TrezorBCH, BlockExplorerCash, BitcoinComCashExplorer,
                BitpayInsightBCH, BlockDozer
            ],
            'unspent_outputs': [
                BlockDozer, TrezorBCH, BlockExplorerCash, BitcoinComCashExplorer,
                BitpayInsightBCH
            ],
            'get_block': [
                BitpayInsightBCH, BitcoinComCashExplorer, BlockDozer, TrezorBCH,
                BlockExplorerCash
            ],
            "get_optimal_fee": []
        },
    },
    'leo': {
        'name': 'LEOcoin',
        'address_version_byte': 18,
        'bip44_coin_type': 0x8000007f,
        'private_key_prefix': 144,
        'script_hash_byte': 88,
        'message_magic': b"\x64\x6e\x78\x82",
        'genesis_date': datetime(2017, 5, 6, 11, 35, 28),
        "github_link": "https://github.com/Leocoin-project/LEOcoin",
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        "seed_nodes": [
            "dnsseed.leocoin.org",
            "leoseed.leocoin.org",
        ] + ["node%s.leocoin.org" % x for x in range(1, 11)],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [LiveCoin, BitZ], 'ltc': [], 'usd': [CCex, LiveCoin], 'doge': []
            },
            'address_balance': [LeoCoinInsight],
            'historical_transactions': [LeoCoinInsight],
            'single_transaction': [LeoCoinInsight],
            'push_tx': [LeoCoinInsight],
            'unspent_outputs': [LeoCoinInsight],
            'get_block': [LeoCoinInsight],
            'get_optimal_fee': []
        },
    },
    'game': {
        'name': 'GameCredits',
        'address_version_byte': 38,
        'bip44_coin_type': 0x80000065,
        'private_key_prefix': 166,
        'script_hash_byte': 5,
        'message_magic': b"\xfb\xc0\xb6\xdb",
        'genesis_date': datetime(1, 1, 1),
        "github_link": "https://github.com/gamecredits-project/GameCredits",
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        "seed_nodes": [
            "dns.gmcseed1.acidpool.com",
            "dns.gmcseed2.version2.org",
            "gmc.cryptocloudhosting.org",
            "gmcnode1.version2.org"
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [
                    Poloniex, Bittrex, LiveCoin, Cryptopia, YoBit, NovaExchange,
                    BitZ, CoinEgg
                ], 'ltc': [Cryptopia], 'usd': [], 'doge': [Cryptopia, NovaExchange]
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'unify': {
        'name': 'Unify',
        'address_version_byte': 68,
        'bip44_coin_type': 0x8000007c,
        'private_key_prefix': 196,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        "github_link": "https://github.com/SBDomains/unify-source",
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Cryptopia, NovaExchange], 'moon': [NovaExchange],
                'ltc': [Cryptopia, NovaExchange], 'usd': [],
                'doge': [Cryptopia, NovaExchange], 'eth': [NovaExchange],
            },
            'address_balance': [UnifyIquidus],
            'historical_transactions': [],
            'single_transaction': [UnifyIquidus],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [UnifyIquidus],
            'get_optimal_fee': []
        },
    },
    'dgb': {
        'name': 'Digibyte',
        'address_version_byte': 30,
        'bip44_coin_type': 0x80000014,
        'private_key_prefix': 128,
        'script_hash_byte': 5,
        'message_magic': b"\xfa\xc3\xb6\xda",
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        "seed_nodes": [
            "seed.digibyte.co",
            "digiexplorer.info",
            "digihash.co"
        ],
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [BitZ], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'zcl': {
        'name': 'ZClassic',
        'address_version_byte': 7352,
        'bip44_coin_type': 0x80000093,
        'private_key_prefix': 128,
        'script_hash_byte': 0x1cbd,
        'genesis_date': datetime(2016, 10, 28, 7, 56),
        'message_magic': b"\x24\xe9\x27\x64",
        'github_link': "https://github.com/z-classic/zclassic",
        'header_hash_algo': 'double-sha256',
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'seed_nodes': [
            'dnsseed.zclassic.org',
            'dnsseed.indieonion.org',
            'dnsseed.rotorproject.org'
        ],
        'port': 8233,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, CCex, Cryptopia],
                'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'xlm': {
        'name': 'Stellar',
        'address_version_byte': None,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Poloniex, Kraken],
                'usd': [Poloniex], 'eth': [Bittrex]
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'b2x': {
        'name': 'Bitcoin 2x',
        'address_version_byte': None,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [HitBTC, YoBit], 'usd': [HitBTC]
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'xem': {
        'name': 'Nem',
        'address_version_byte': None,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': None,
        'script_hash_algo': None,
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, Poloniex, Cryptopia, HitBTC, LiveCoin, YoBit],
                'usd': [YoBit, HitBTC, LiveCoin], 'eth': [Bittrex, HitBTC, LiveCoin, YoBit],
                'doge': [Cryptopia, YoBit], 'rur': [YoBit], 'ltc': [Cryptopia]
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'btg': {
        'name': 'Bitcoin Gold',
        'address_version_byte': 38,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [
                    Bittrex, BitFinex, BleuTrade, CexIO, Cryptopia, HitBTC,
                    KuCoin, NovaExchange, YoBit, OKEX, BitZ, EXX
                ],
                'ltc': [Cryptopia], 'krw': [CoinNest],
                'usd': [BitFinex, Bittrex, CexIO, HitBTC, YoBit], 'eur': [CexIO],
                'eth': [Bittrex, BleuTrade, HitBTC, KuCoin, NovaExchange, YoBit],
                'doge': [BleuTrade, Cryptopia, NovaExchange, YoBit], 'rur': [YoBit]
            },
            'address_balance': [Btgexp],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
    'mona': {
        'name': 'MonaCoin',
        'address_version_byte': 50,
        'bip44_coin_type': 0x80000016,
        'private_key_prefix': 176,
        'script_hash_byte': 55,
        'message_magic': None,
        'genesis_date': datetime(2013, 12, 31, 8, 44),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex, BleuTrade, BitBank], 'ltc': [], 'usd': [],
                'jpy': [BitBank, Zaif], 'qtum': [EXX], 'hsr': [EXX]
            },
            'address_balance': [MonaInsight],
            'historical_transactions': [MonaInsight],
            'single_transaction': [MonaInsight],
            'push_tx': [MonaInsight],
            'unspent_outputs': [MonaInsight],
            'get_block': [MonaInsight],
            'get_optimal_fee': []
        },
    },
    'btcp': {
        'name': 'Bitcoin Private',
        'address_version_byte': 19,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [Bittrex], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },

    # TEMPLATE
    '': {
        'name': '',
        'address_version_byte': None,
        'bip44_coin_type': None,
        'private_key_prefix': None,
        'script_hash_byte': None,
        'message_magic': None,
        'genesis_date': datetime(1, 1, 1),
        'github_link': None,
        'header_hash_algo': None,
        'transaction_hash_algo': 'double-sha256',
        'script_hash_algo': 'double-sha256',
        'port': None,
        'supply_data': {
            'method': 'standard',
            'start_coins_per_block': None,
            'minutes_per_block': None,
            'full_cap': None,
            'blocks_per_era': None,
            'reward_ends_at_block': None
        },
        'services': {
            'current_price': {
                'btc': [], 'ltc': [], 'usd': [], 'doge': []
            },
            'address_balance': [],
            'historical_transactions': [],
            'single_transaction': [],
            'push_tx': [],
            'unspent_outputs': [],
            'get_block': [],
            'get_optimal_fee': []
        },
    },
}
